@@ -32,13 +32,29 @@ pub const APP_LAYER_EVENT_TYPE_TRANSACTION : i32 = 1;
 pub const APP_LAYER_EVENT_TYPE_PACKET      : i32 = 2;
 
 // From stream.h.
+pub const STREAM_START:    u8 = 0x01;
+pub const STREAM_EOF:      u8 = 0x02;
 pub const STREAM_TOSERVER: u8 = 0x04;
 pub const STREAM_TOCLIENT: u8 = 0x08;
+pub const STREAM_GAP:      u8 = 0x10;
+pub const STREAM_DEPTH:    u8 = 0x20;
+pub const STREAM_MIDSTREAM:u8 = 0x40;
+
+// Application layer protocol identifiers (app-layer-protos.h)
+pub type AppProto = libc::c_int;
+
+pub const ALPROTO_UNKNOWN : AppProto = 0;
+pub static mut ALPROTO_FAILED : AppProto = 0; // updated during init
 
 macro_rules!BIT_U64 {
     ($x:expr) => (1 << $x);
 }
 
+// Defined in app-layer-protos.h
+extern {
+    pub fn StringToAppProto(proto_name: *const u8) -> AppProto;
+}
+
 //
 // Function types for calls into C.
 //
@@ -130,6 +146,7 @@ pub extern "C" fn rs_init(context: &'static mut SuricataContext)
 {
     unsafe {
         SC = Some(context);
+        ALPROTO_FAILED = StringToAppProto("failed\0".as_ptr());
     }
 }
 
