@@ -39,12 +39,12 @@
 #define UNIFIED2_IDS_EVENT_IPV6_MPLS_TYPE 100
 #define UNIFIED2_IDS_EVENT_EXTRADATA_TYPE 110
 #define UNIFIED2_EXTRADATA_CLIENT_IPV4_TYPE 1
-#define UNIFIED2_EXTRADATA_CLIENT_IPV6_TYPE 1
+#define UNIFIED2_EXTRADATA_CLIENT_IPV6_TYPE 2
 #define UNIFIED2_EXTRADATA_TYPE_BLOB 1
 #define UNIFIED2_EXTRADATA_TYPE_EXTRA_DATA 4
 
 void Unified2AlertRegister(void);
-OutputCtx *Unified2AlertInitCtx(ConfNode *);
+OutputInitResult Unified2AlertInitCtx(ConfNode *);
 
 #endif /* __ALERT_UNIFIED2_ALERT_H__ */
 
