@@ -42,8 +42,8 @@
 static pcre *regex = NULL;
 static pcre_extra *regex_study = NULL;
 
-static int DetectClasstypeSetup(DetectEngineCtx *, Signature *, char *);
-void DetectClasstypeRegisterTests(void);
+static int DetectClasstypeSetup(DetectEngineCtx *, Signature *, const char *);
+static void DetectClasstypeRegisterTests(void);
 
 /**
  * \brief Registers the handler functions for the "Classtype" keyword.
@@ -68,20 +68,13 @@ void DetectClasstypeRegister(void)
  *
  * \retval bool success or failure.
  */
-static int DetectClasstypeParseRawString(char *rawstr, char *out, size_t outsize)
+static int DetectClasstypeParseRawString(const char *rawstr, char *out, size_t outsize)
 {
 #define MAX_SUBSTRINGS 30
     int ret = 0;
     int ov[MAX_SUBSTRINGS];
     size_t len = strlen(rawstr);
 
-    /* get rid of the double quotes if present */
-    if (rawstr[0] == '\"' && rawstr[strlen(rawstr) - 1] == '\"') {
-        rawstr++;
-        rawstr[strlen(rawstr) - 1] = '\0';
-        len -= 2;
-    }
-
     ret = pcre_exec(regex, regex_study, rawstr, len, 0, 0, ov, 30);
     if (ret < 0) {
         SCLogError(SC_ERR_PCRE_MATCH, "Invalid Classtype in Signature");
@@ -110,7 +103,7 @@ static int DetectClasstypeParseRawString(char *rawstr, char *out, size_t outsize
  * \retval  0 On success
  * \retval -1 On failure
  */
-static int DetectClasstypeSetup(DetectEngineCtx *de_ctx, Signature *s, char *rawstr)
+static int DetectClasstypeSetup(DetectEngineCtx *de_ctx, Signature *s, const char *rawstr)
 {
     char parsed_ct_name[1024] = "";
     SCClassConfClasstype *ct = NULL;
@@ -128,6 +121,12 @@ static int DetectClasstypeSetup(DetectEngineCtx *de_ctx, Signature *s, char *raw
         goto error;
     }
 
+    if ((s->class > 0) || (s->class_msg != NULL))
+    {
+        SCLogError(SC_ERR_INVALID_RULE_ARGUMENT, "duplicated 'classtype' keyword detected");
+        goto error;
+    }
+
     /* if we have retrieved the classtype, assign the message to be displayed
      * for this Signature by fast.log, if a Packet matches this Signature */
     s->class = ct->classtype_id;
@@ -153,7 +152,7 @@ static int DetectClasstypeSetup(DetectEngineCtx *de_ctx, Signature *s, char *raw
  * \test Check that supplying an invalid classtype in the rule, results in the
  *       rule being invalidated.
  */
-int DetectClasstypeTest01()
+static int DetectClasstypeTest01(void)
 {
     int result = 0;
 
@@ -182,7 +181,7 @@ end:
  *       properly, with rules containing invalid classtypes being rejected
  *       and the ones containing valid classtypes parsed and returned.
  */
-int DetectClasstypeTest02()
+static int DetectClasstypeTest02(void)
 {
     int result = 0;
     Signature *last = NULL;
@@ -248,7 +247,7 @@ end:
  * \test Check that the signatures are assigned priority based on classtype they
  *       are given.
  */
-int DetectClasstypeTest03()
+static int DetectClasstypeTest03(void)
 {
     int result = 0;
     Signature *last = NULL;
@@ -308,7 +307,7 @@ end:
 /**
  * \brief This function registers unit tests for Classification Config API.
  */
-void DetectClasstypeRegisterTests(void)
+static void DetectClasstypeRegisterTests(void)
 {
 
 #ifdef UNITTESTS
