@@ -19,6 +19,7 @@
  * \file
  *
  * \author Anoop Saldanha <anoopsaldanha@gmail.com>
+ * \author Victor Julien <victor@inliniac.net>
  */
 
 #ifndef __DETECT_ENGINE_CONTENT_INSPECTION_H__
@@ -33,11 +34,19 @@ enum {
     DETECT_ENGINE_CONTENT_INSPECTION_MODE_STATE,
 };
 
+#define DETECT_CI_FLAGS_START   BIT_U8(0)   /**< unused, reserved for future use */
+#define DETECT_CI_FLAGS_END     BIT_U8(1)   /**< indication that current buffer
+                                             *   is the end of the data */
+
+/** buffer is a single, non-streaming, buffer. Data sent to the content
+ *  inspection function contains both start and end of the data. */
+#define DETECT_CI_FLAGS_SINGLE  (DETECT_CI_FLAGS_START|DETECT_CI_FLAGS_END)
+
 int DetectEngineContentInspection(DetectEngineCtx *de_ctx, DetectEngineThreadCtx *det_ctx,
                                   const Signature *s, const SigMatchData *smd,
                                   Flow *f,
                                   uint8_t *buffer, uint32_t buffer_len,
-                                  uint32_t stream_start_offset,
+                                  uint32_t stream_start_offset, uint8_t flags,
                                   uint8_t inspection_mode, void *data);
 
 void DetectEngineContentInspectionRegisterTests(void);
