@@ -65,18 +65,19 @@
 
 #include "util-validate.h"
 
-#define BUFFER_GROW_STEP 50
+#define BUFFER_STEP 50
 
 static inline int HCBDCreateSpace(DetectEngineThreadCtx *det_ctx, uint64_t size)
 {
-    if (size >= (USHRT_MAX - BUFFER_GROW_STEP))
+    if (size >= (USHRT_MAX - BUFFER_STEP))
         return -1;
 
+    void *ptmp;
     if (size > det_ctx->hcbd_buffers_size) {
         uint16_t grow_by = size - det_ctx->hcbd_buffers_size;
-        grow_by = MAX(grow_by, BUFFER_GROW_STEP);
+        grow_by = MAX(grow_by, BUFFER_STEP);
 
-        void *ptmp = SCRealloc(det_ctx->hcbd,
+        ptmp = SCRealloc(det_ctx->hcbd,
                          (det_ctx->hcbd_buffers_size + grow_by) * sizeof(HttpReassembledBody));
         if (ptmp == NULL) {
             SCFree(det_ctx->hcbd);
@@ -250,11 +251,12 @@ static void PrefilterTxHttpRequestBody(DetectEngineThreadCtx *det_ctx,
     }
 }
 
-int PrefilterTxHttpRequestBodyRegister(SigGroupHead *sgh, MpmCtx *mpm_ctx)
+int PrefilterTxHttpRequestBodyRegister(DetectEngineCtx *de_ctx,
+        SigGroupHead *sgh, MpmCtx *mpm_ctx)
 {
     SCEnter();
 
-    return PrefilterAppendTxEngine(sgh, PrefilterTxHttpRequestBody,
+    return PrefilterAppendTxEngine(de_ctx, sgh, PrefilterTxHttpRequestBody,
         ALPROTO_HTTP, HTP_REQUEST_BODY,
         mpm_ctx, NULL, "http_client_body");
 }
@@ -267,6 +269,7 @@ int DetectEngineInspectHttpClientBody(ThreadVars *tv,
     HtpState *htp_state = (HtpState *)alstate;
     uint32_t buffer_len = 0;
     uint32_t stream_start_offset = 0;
+    const bool eof = (AppLayerParserGetStateProgress(IPPROTO_TCP, ALPROTO_HTTP, tx, flags) > HTP_REQUEST_BODY);
     const uint8_t *buffer = DetectEngineHCBDGetBufferForTX(tx, tx_id,
                                                      de_ctx, det_ctx,
                                                      f, htp_state,
@@ -276,6 +279,9 @@ int DetectEngineInspectHttpClientBody(ThreadVars *tv,
     if (buffer_len == 0)
         goto end;
 
+    uint8_t ci_flags = eof ? DETECT_CI_FLAGS_END : 0;
+    ci_flags |= (stream_start_offset == 0 ? DETECT_CI_FLAGS_START : 0);
+
     det_ctx->buffer_offset = 0;
     det_ctx->discontinue_matching = 0;
     det_ctx->inspection_recursion_counter = 0;
@@ -283,14 +289,14 @@ int DetectEngineInspectHttpClientBody(ThreadVars *tv,
                                           f,
                                           (uint8_t *)buffer,
                                           buffer_len,
-                                          stream_start_offset,
+                                          stream_start_offset, ci_flags,
                                           DETECT_ENGINE_CONTENT_INSPECTION_MODE_STATE, NULL);
     if (r == 1)
         return DETECT_ENGINE_INSPECT_SIG_MATCH;
 
 
  end:
-    if (AppLayerParserGetStateProgress(IPPROTO_TCP, ALPROTO_HTTP, tx, flags) > HTP_REQUEST_BODY)
+    if (eof)
         return DETECT_ENGINE_INSPECT_SIG_CANT_MATCH;
     else
         return DETECT_ENGINE_INSPECT_SIG_NO_MATCH;
