@@ -375,7 +375,7 @@ static void TagHandlePacketFlow(Flow *f, Packet *p)
     }
 }
 
-void TagHandlePacketHost(Host *host, Packet *p)
+static void TagHandlePacketHost(Host *host, Packet *p)
 {
     DetectTagDataEntry *tde = NULL;
     DetectTagDataEntry *prev = NULL;
@@ -591,7 +591,7 @@ int TagTimeoutCheck(Host *host, struct timeval *tv)
 /**
  * \test host tagging: packets
  */
-int DetectTagTestPacket01 (void)
+static int DetectTagTestPacket01 (void)
 {
     int result = 0;
     uint8_t *buf = (uint8_t *)"Hi all!";
@@ -622,7 +622,7 @@ int DetectTagTestPacket01 (void)
                               "192.168.1.5", "192.168.1.11",
                               41424, 80);
 
-    char *sigs[5];
+    const char *sigs[5];
     sigs[0]= "alert tcp any any -> any any (msg:\"Testing tag 1\"; content:\"Hi all\"; tag:host,3,packets,src; sid:1;)";
     sigs[1]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"Hi all\"; tag:host,4,packets,dst; sid:2;)";
     sigs[2]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"no match\"; sid:3;)";
@@ -688,7 +688,7 @@ int DetectTagTestPacket01 (void)
 /**
  * \test host tagging: seconds
  */
-int DetectTagTestPacket02 (void)
+static int DetectTagTestPacket02 (void)
 {
     int result = 0;
     uint8_t *buf = (uint8_t *)"Hi all!";
@@ -736,7 +736,7 @@ int DetectTagTestPacket02 (void)
                               "192.168.1.5", "192.168.1.11",
                               41424, 80);
 
-    char *sigs[5];
+    const char *sigs[5];
     sigs[0]= "alert tcp any any -> any any (msg:\"Testing tag 1\"; content:\"Hi all\"; tag:host,3,seconds,src; sid:1;)";
     sigs[1]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"Hi all\"; tag:host,8,seconds,dst; sid:2;)";
     sigs[2]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"no match\"; sid:3;)";
@@ -857,7 +857,7 @@ static int DetectTagTestPacket03 (void)
                               "192.168.1.5", "192.168.1.11",
                               41424, 80);
 
-    char *sigs[5];
+    const char *sigs[5];
     sigs[0]= "alert tcp any any -> any any (msg:\"Testing tag 1\"; content:\"Hi all\"; tag:host, 150, bytes, src; sid:1;)";
     sigs[1]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"Hi all\"; tag:host, 150, bytes, dst; sid:2;)";
     sigs[2]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"no match\"; sid:3;)";
@@ -991,7 +991,7 @@ static int DetectTagTestPacket04 (void)
                               "192.168.1.5", "192.168.1.1",
                               80, 41424);
 
-    char *sigs[5];
+    const char *sigs[5];
     sigs[0]= "alert tcp any any -> any any (msg:\"Testing tag 1\"; content:\"Hi all\"; tag:session,4,packets; sid:1;)";
     sigs[1]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"blahblah\"; sid:2;)";
     sigs[2]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"no match\"; sid:3;)";
@@ -1132,7 +1132,7 @@ static int DetectTagTestPacket05 (void)
                               "192.168.1.5", "192.168.1.1",
                               80, 41424);
 
-    char *sigs[5];
+    const char *sigs[5];
     sigs[0]= "alert tcp any any -> any any (msg:\"Testing tag 1\"; content:\"Hi all\"; tag:session,8,seconds; sid:1;)";
     sigs[1]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"blahblah\"; sid:2;)";
     sigs[2]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"no match\"; sid:3;)";
@@ -1278,7 +1278,7 @@ static int DetectTagTestPacket06 (void)
                               "192.168.1.5", "192.168.1.1",
                               80, 41424);
 
-    char *sigs[5];
+    const char *sigs[5];
     sigs[0]= "alert tcp any any -> any any (msg:\"Testing tag 1\"; content:\"Hi all\"; tag:session,150,bytes; sid:1;)";
     sigs[1]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"blahblah\"; sid:2;)";
     sigs[2]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"no match\"; sid:3;)";
@@ -1420,7 +1420,7 @@ static int DetectTagTestPacket07 (void)
                               "192.168.1.5", "192.168.1.1",
                               80, 41424);
 
-    char *sigs[5];
+    const char *sigs[5];
     sigs[0]= "alert tcp any any -> any any (msg:\"Testing tag 1\"; content:\"Hi all\"; tag:session,150,bytes; sid:1;)";
     sigs[1]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"blahblah\"; sid:2;)";
     sigs[2]= "alert tcp any any -> any any (msg:\"Testing tag 2\"; content:\"no match\"; sid:3;)";
