@@ -42,8 +42,9 @@
 static pcre *parse_regex;
 static pcre_extra *parse_regex_study;
 
-static int DetectMarkSetup (DetectEngineCtx *, Signature *, char *);
-int DetectMarkPacket(ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p, Signature *s, const SigMatchCtx *ctx);
+static int DetectMarkSetup (DetectEngineCtx *, Signature *, const char *);
+static int DetectMarkPacket(ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p,
+        const Signature *s, const SigMatchCtx *ctx);
 void DetectMarkDataFree(void *ptr);
 
 /**
@@ -71,7 +72,7 @@ void DetectMarkRegister (void)
  * \retval 0 on success
  * \retval < 0 on failure
  */
-static void * DetectMarkParse (char *rawstr)
+static void * DetectMarkParse (const char *rawstr)
 {
     int ret = 0, res = 0;
 #define MAX_SUBSTRINGS 30
@@ -181,7 +182,7 @@ static void * DetectMarkParse (char *rawstr)
  * \retval 0 on Success
  * \retval -1 on Failure
  */
-static int DetectMarkSetup (DetectEngineCtx *de_ctx, Signature *s, char *rawstr)
+static int DetectMarkSetup (DetectEngineCtx *de_ctx, Signature *s, const char *rawstr)
 {
 #ifdef NFQ
     DetectMarkData *data = NULL;
@@ -217,7 +218,8 @@ void DetectMarkDataFree(void *ptr)
 }
 
 
-int DetectMarkPacket(ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p, Signature *s, const SigMatchCtx *ctx)
+static int DetectMarkPacket(ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p,
+        const Signature *s, const SigMatchCtx *ctx)
 {
 #ifdef NFQ
     const DetectMarkData *nf_data = (const DetectMarkData *)ctx;
