@@ -34,6 +34,8 @@
 #include "detect-pcre.h"
 #include "detect-nfs-procedure.h"
 
+#include "app-layer-parser.h"
+
 #include "flow.h"
 #include "flow-util.h"
 #include "flow-var.h"
@@ -188,7 +190,7 @@ static int DetectNfsProcedureMatch (ThreadVars *t, DetectEngineThreadCtx *det_ct
     uint16_t i;
     for (i = 0; i < 256; i++) {
         uint32_t procedure;
-        if (rs_nfs3_tx_get_procedures(txv, i, &procedure) == 1) {
+        if (rs_nfs_tx_get_procedures(txv, i, &procedure) == 1) {
             SCLogDebug("proc %u mode %u lo %u hi %u",
                     procedure, dd->mode, dd->lo, dd->hi);
             if (ProcedureMatch(procedure, dd->mode, dd->lo, dd->hi))
@@ -362,7 +364,6 @@ static int DetectNfsProcedureSetup (DetectEngineCtx *de_ctx, Signature *s,
     sm->type = DETECT_AL_NFS_PROCEDURE;
     sm->ctx = (void *)dd;
 
-    s->flags |= SIG_FLAG_STATE_MATCH;
     SCLogDebug("low %u hi %u", dd->lo, dd->hi);
     SigMatchAppendSMToList(s, sm, g_nfs_request_buffer_id);
     return 0;
