@@ -39,6 +39,8 @@
 #include "detect.h"
 #include "detect-engine-state.h"
 
+#include "decode-icmpv4.h"
+
 /** \brief allocate a flow
  *
  *  We check against the memuse counter. If it passes that check we increment
@@ -122,6 +124,24 @@ uint8_t FlowGetReverseProtoMapping(uint8_t rproto)
     }
 }
 
+static inline void FlowSetICMPv4CounterPart(Flow *f)
+{
+    int ctype = ICMPv4GetCounterpart(f->icmp_s.type);
+    if (ctype == -1)
+        return;
+
+    f->icmp_d.type = (uint8_t)ctype;
+}
+
+static inline void FlowSetICMPv6CounterPart(Flow *f)
+{
+    int ctype = ICMPv6GetCounterpart(f->icmp_s.type);
+    if (ctype == -1)
+        return;
+
+    f->icmp_d.type = (uint8_t)ctype;
+}
+
 /* initialize the flow from the first packet
  * we see from it. */
 void FlowInit(Flow *f, const Packet *p)
@@ -137,10 +157,12 @@ void FlowInit(Flow *f, const Packet *p)
     if (PKT_IS_IPV4(p)) {
         FLOW_SET_IPV4_SRC_ADDR_FROM_PACKET(p, &f->src);
         FLOW_SET_IPV4_DST_ADDR_FROM_PACKET(p, &f->dst);
+        f->min_ttl_toserver = f->max_ttl_toserver = IPV4_GET_IPTTL((p));
         f->flags |= FLOW_IPV4;
     } else if (PKT_IS_IPV6(p)) {
         FLOW_SET_IPV6_SRC_ADDR_FROM_PACKET(p, &f->src);
         FLOW_SET_IPV6_DST_ADDR_FROM_PACKET(p, &f->dst);
+        f->min_ttl_toserver = f->max_ttl_toserver = IPV6_GET_HLIM((p));
         f->flags |= FLOW_IPV6;
     }
 #ifdef DEBUG
@@ -157,11 +179,13 @@ void FlowInit(Flow *f, const Packet *p)
         SET_UDP_SRC_PORT(p,&f->sp);
         SET_UDP_DST_PORT(p,&f->dp);
     } else if (p->icmpv4h != NULL) {
-        f->type = p->type;
-        f->code = p->code;
+        f->icmp_s.type = p->icmp_s.type;
+        f->icmp_s.code = p->icmp_s.code;
+        FlowSetICMPv4CounterPart(f);
     } else if (p->icmpv6h != NULL) {
-        f->type = p->type;
-        f->code = p->code;
+        f->icmp_s.type = p->icmp_s.type;
+        f->icmp_s.code = p->icmp_s.code;
+        FlowSetICMPv6CounterPart(f);
     } else if (p->sctph != NULL) { /* XXX MACRO */
         SET_SCTP_SRC_PORT(p,&f->sp);
         SET_SCTP_DST_PORT(p,&f->dp);
