@@ -63,6 +63,7 @@
 #define MODULE_NAME "LogFilestoreLog"
 
 static char g_logfile_base_dir[PATH_MAX] = "/tmp";
+static char g_working_file_suffix[PATH_MAX] = ".tmp";
 
 SC_ATOMIC_DECLARE(uint32_t, filestore_open_file_cnt);  /**< Atomic counter of simultaneously open files */
 
@@ -203,13 +204,25 @@ static uint32_t FileGetMaxOpenFiles(void)
     return g_file_store_max_open_files;
 }
 
-static void LogFilestoreLogCreateMetaFile(const Packet *p, const File *ff, char *filename, int ipver) {
+static int g_file_store_include_pid = 0;
+
+static void FileIncludePidEnable(void)
+{
+    g_file_store_include_pid = 1;
+}
+
+static int FileIncludePid(void)
+{
+    return g_file_store_include_pid;
+}
+
+static void LogFilestoreLogCreateMetaFile(const Packet *p, const File *ff, char *base_filename, int ipver) {
     if (!FileWriteMeta())
         return;
 
     char metafilename[PATH_MAX] = "";
-    if (snprintf(metafilename, sizeof(metafilename), "%s.meta",
-                filename) == sizeof(metafilename))
+    if (snprintf(metafilename, sizeof(metafilename), "%s.meta%s", base_filename,
+            g_working_file_suffix) == sizeof(metafilename))
         return;
 
     FILE *fp = fopen(metafilename, "w+");
@@ -282,18 +295,23 @@ static void LogFilestoreLogCreateMetaFile(const Packet *p, const File *ff, char
 
 static void LogFilestoreLogCloseMetaFile(const File *ff)
 {
-    if (!FileWriteMeta())
+    char pid_expression[PATH_MAX] = "";
+    if (FileIncludePid())
+        snprintf(pid_expression, sizeof(pid_expression), ".%d", getpid());
+    char final_filename[PATH_MAX] = "";
+    if (snprintf(final_filename, sizeof(final_filename), "%s/file%s.%u",
+            g_logfile_base_dir, pid_expression, ff->file_store_id) == sizeof(final_filename))
+        return;
+    char final_metafilename[PATH_MAX] = "";
+    if (snprintf(final_metafilename, sizeof(final_metafilename),
+            "%s.meta", final_filename) == sizeof(final_metafilename))
+        return;
+    char working_metafilename[PATH_MAX] = "";
+    if (snprintf(working_metafilename, sizeof(working_metafilename),
+            "%s%s", final_metafilename, g_working_file_suffix) == sizeof(working_metafilename))
         return;
 
-    char filename[PATH_MAX] = "";
-    if (snprintf(filename, sizeof(filename), "%s/file.%u",
-            g_logfile_base_dir, ff->file_store_id) == sizeof(filename))
-        return;
-    char metafilename[PATH_MAX] = "";
-    if (snprintf(metafilename, sizeof(metafilename), "%s.meta",
-                filename) == sizeof(metafilename))
-        return;
-    FILE *fp = fopen(metafilename, "a");
+    FILE *fp = fopen(working_metafilename, "a");
     if (fp != NULL) {
 #ifdef HAVE_MAGIC
         fprintf(fp, "MAGIC:             %s\n",
@@ -343,12 +361,50 @@ static void LogFilestoreLogCloseMetaFile(const File *ff)
 
         fclose(fp);
     } else {
-        SCLogInfo("opening %s failed: %s", metafilename, strerror(errno));
+        SCLogInfo("opening %s failed: %s", working_metafilename,
+                strerror(errno));
+    }
+}
+
+static void LogFilestoreFinalizeFiles(const File *ff) {
+    char pid_expression[PATH_MAX] = "";
+    if (FileIncludePid())
+        snprintf(pid_expression, sizeof(pid_expression), ".%d", getpid());
+    char final_filename[PATH_MAX] = "";
+    if (snprintf(final_filename, sizeof(final_filename), "%s/file%s.%u",
+            g_logfile_base_dir, pid_expression, ff->file_store_id) == sizeof(final_filename))
+        return;
+    char working_filename[PATH_MAX] = "";
+    if (snprintf(working_filename, sizeof(working_filename), "%s%s",
+            final_filename, g_working_file_suffix) == sizeof(working_filename))
+           return;
+
+    if (rename(working_filename, final_filename) != 0) {
+        SCLogWarning(SC_WARN_RENAMING_FILE, "renaming file %s to %s failed",
+                working_filename, final_filename);
+        return;
+    }
+    if (FileWriteMeta()) {
+        LogFilestoreLogCloseMetaFile(ff);
+        char final_metafilename[PATH_MAX] = "";
+        if (snprintf(final_metafilename, sizeof(final_metafilename),
+                "%s.meta", final_filename) == sizeof(final_metafilename))
+            return;
+        char working_metafilename[PATH_MAX] = "";
+        if (snprintf(working_metafilename, sizeof(working_metafilename),
+                "%s%s", final_metafilename, g_working_file_suffix) == sizeof(working_metafilename))
+            return;
+
+        if (rename(working_metafilename, final_metafilename) != 0) {
+            SCLogWarning(SC_WARN_RENAMING_FILE,
+                    "renaming metafile %s to %s failed", working_metafilename,
+                    final_metafilename);
+        }
     }
 }
 
 static int LogFilestoreLogger(ThreadVars *tv, void *thread_data, const Packet *p,
-        File *ff, const uint8_t *data, uint32_t data_len, uint8_t flags)
+        File *ff, const uint8_t *data, uint32_t data_len, uint8_t flags, uint8_t dir)
 {
     SCEnter();
     LogFilestoreLogThread *aft = (LogFilestoreLogThread *)thread_data;
@@ -371,15 +427,22 @@ static int LogFilestoreLogger(ThreadVars *tv, void *thread_data, const Packet *p
 
     SCLogDebug("ff %p, data %p, data_len %u", ff, data, data_len);
 
-    if (snprintf(filename, sizeof(filename), "%s/file.%u",
-            g_logfile_base_dir, ff->file_store_id) == sizeof(filename))
+    char pid_expression[PATH_MAX] = "";
+    if (FileIncludePid())
+        snprintf(pid_expression, sizeof(pid_expression), ".%d", getpid());
+    char base_filename[PATH_MAX] = "";
+    if (snprintf(base_filename, sizeof(base_filename), "%s/file%s.%u",
+            g_logfile_base_dir, pid_expression, ff->file_store_id) == sizeof(base_filename))
+        return -1;
+    if (snprintf(filename, sizeof(filename), "%s%s", base_filename,
+            g_working_file_suffix) == sizeof(filename))
         return -1;
 
     if (flags & OUTPUT_FILEDATA_FLAG_OPEN) {
         aft->file_cnt++;
 
         /* create a .meta file that contains time, src/dst/sp/dp/proto */
-        LogFilestoreLogCreateMetaFile(p, ff, filename, ipver);
+        LogFilestoreLogCreateMetaFile(p, ff, base_filename, ipver);
 
         if (SC_ATOMIC_GET(filestore_open_file_cnt) < FileGetMaxOpenFiles()) {
             SC_ATOMIC_ADD(filestore_open_file_cnt, 1);
@@ -433,7 +496,7 @@ static int LogFilestoreLogger(ThreadVars *tv, void *thread_data, const Packet *p
             ff->fd = -1;
             SC_ATOMIC_SUB(filestore_open_file_cnt, 1);
         }
-        LogFilestoreLogCloseMetaFile(ff);
+        LogFilestoreFinalizeFiles(ff);
     }
 
     return 0;
@@ -459,7 +522,8 @@ static TmEcode LogFilestoreLogThreadInit(ThreadVars *t, const void *initdata, vo
     struct stat stat_buf;
     if (stat(g_logfile_base_dir, &stat_buf) != 0) {
         int ret;
-        ret = mkdir(g_logfile_base_dir, S_IRWXU|S_IXGRP|S_IRGRP);
+        // coverity[toctou : FALSE]
+        ret = SCMkDir(g_logfile_base_dir, S_IRWXU|S_IXGRP|S_IRGRP);
         if (ret != 0) {
             int err = errno;
             if (err != EEXIST) {
@@ -524,11 +588,28 @@ static void LogFilestoreLogDeInitCtx(OutputCtx *output_ctx)
  *  \param conf Pointer to ConfNode containing this loggers configuration.
  *  \return NULL if failure, LogFilestoreCtx* to the file_ctx if succesful
  * */
-static OutputCtx *LogFilestoreLogInitCtx(ConfNode *conf)
+static OutputInitResult LogFilestoreLogInitCtx(ConfNode *conf)
 {
+    OutputInitResult result = { NULL, false };
+
+    intmax_t version = 0;
+    if (ConfGetChildValueInt(conf, "version", &version)) {
+        if (version > 1) {
+            result.ok = true;
+            return result;
+        }
+    }
+
+    if (RunModeOutputFiledataEnabled()) {
+        SCLogWarning(SC_ERR_NOT_SUPPORTED,
+                "A file data logger is already enabled. Filestore (v1) "
+                "will not be enabled.");
+        return result;
+    }
+
     OutputCtx *output_ctx = SCCalloc(1, sizeof(OutputCtx));
     if (unlikely(output_ctx == NULL))
-        return NULL;
+        return result;
 
     output_ctx->data = NULL;
     output_ctx->DeInit = LogFilestoreLogDeInitCtx;
@@ -606,16 +687,20 @@ static OutputCtx *LogFilestoreLogInitCtx(ConfNode *conf)
         }
     }
 
-    SCReturnPtr(output_ctx, "OutputCtx");
+    const char *include_pid = ConfNodeLookupChildValue(conf, "include-pid");
+    if (include_pid != NULL && ConfValIsTrue(include_pid)) {
+        FileIncludePidEnable();
+        SCLogInfo("enabling pid as a part of all file names");
+    }
+
+    StatsRegisterGlobalCounter("file_store.open_files",
+            LogFilestoreOpenFilesCounter);
+
+    result.ctx = output_ctx;
+    result.ok = true;
+    SCReturnCT(result, "OutputInitResult");
 }
 
-
-void LogFilestoreInitConfig(void)
-{
-    StatsRegisterGlobalCounter("file_store.open_files", LogFilestoreOpenFilesCounter);
-}
-
-
 void LogFilestoreRegister (void)
 {
     OutputRegisterFiledataModule(LOGGER_FILE_STORE, MODULE_NAME, "file",
