@@ -1,4 +1,4 @@
-/* Copyright (C) 2011,2012 Open Information Security Foundation
+/* Copyright (C) 2011-2016 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -30,15 +30,14 @@
  *
  */
 
-
 #include "suricata-common.h"
 #include "config.h"
 #include "tm-threads.h"
 #include "conf.h"
 #include "runmodes.h"
 #include "runmode-af-packet.h"
-#include "log-httplog.h"
 #include "output.h"
+#include "log-httplog.h"
 #include "detect-engine-mpm.h"
 
 #include "alert-fastlog.h"
@@ -46,6 +45,8 @@
 #include "alert-unified2-alert.h"
 #include "alert-debuglog.h"
 
+#include "flow-bypass.h"
+
 #include "util-debug.h"
 #include "util-time.h"
 #include "util-cpu.h"
@@ -53,6 +54,7 @@
 #include "util-device.h"
 #include "util-runmodes.h"
 #include "util-ioctl.h"
+#include "util-ebpf.h"
 
 #include "source-af-packet.h"
 
@@ -123,6 +125,7 @@ static void *ParseAFPConfig(const char *iface)
     const char *bpf_filter = NULL;
     const char *out_iface = NULL;
     int cluster_type = PACKET_FANOUT_HASH;
+    const char *ebpf_file = NULL;
 
     if (iface == NULL) {
         return NULL;
@@ -145,6 +148,10 @@ static void *ParseAFPConfig(const char *iface)
     aconf->DerefFunc = AFPDerefConfig;
     aconf->flags = AFP_RING_MODE;
     aconf->bpf_filter = NULL;
+    aconf->ebpf_lb_file = NULL;
+    aconf->ebpf_lb_fd = -1;
+    aconf->ebpf_filter_file = NULL;
+    aconf->ebpf_filter_fd = -1;
     aconf->out_iface = NULL;
     aconf->copy_mode = AFP_COPY_MODE_NONE;
     aconf->block_timeout = 10;
@@ -331,7 +338,13 @@ static void *ParseAFPConfig(const char *iface)
                 aconf->iface);
         aconf->cluster_type = PACKET_FANOUT_ROLLOVER;
         cluster_type = PACKET_FANOUT_ROLLOVER;
-
+#ifdef HAVE_PACKET_EBPF
+    } else if (strcmp(tmpctype, "cluster_ebpf") == 0) {
+        SCLogInfo("Using ebpf based cluster mode for AF_PACKET (iface %s)",
+                aconf->iface);
+        aconf->cluster_type = PACKET_FANOUT_EBPF;
+        cluster_type = PACKET_FANOUT_EBPF;
+#endif
     } else {
         SCLogWarning(SC_ERR_INVALID_CLUSTER_TYPE,"invalid cluster-type %s",tmpctype);
     }
@@ -355,6 +368,145 @@ static void *ParseAFPConfig(const char *iface)
         }
     }
 
+    if (ConfGetChildValueWithDefault(if_root, if_default, "ebpf-lb-file", &ebpf_file) != 1) {
+        aconf->ebpf_lb_file = NULL;
+    } else {
+#ifdef HAVE_PACKET_EBPF
+        SCLogConfig("af-packet will use '%s' as eBPF load balancing file",
+                  ebpf_file);
+#endif
+        aconf->ebpf_lb_file = ebpf_file;
+    }
+
+#ifdef HAVE_PACKET_EBPF
+    /* One shot loading of the eBPF file */
+    if (aconf->ebpf_lb_file && cluster_type == PACKET_FANOUT_EBPF) {
+        int ret = EBPFLoadFile(aconf->iface, aconf->ebpf_lb_file, "loadbalancer",
+                               &aconf->ebpf_lb_fd, EBPF_SOCKET_FILTER);
+        if (ret != 0) {
+            SCLogWarning(SC_ERR_INVALID_VALUE, "Error when loading eBPF lb file");
+        }
+    }
+#else
+    if (aconf->ebpf_lb_file) {
+        SCLogError(SC_ERR_UNIMPLEMENTED, "eBPF support is not build-in");
+    }
+#endif
+
+    if (ConfGetChildValueWithDefault(if_root, if_default, "ebpf-filter-file", &ebpf_file) != 1) {
+        aconf->ebpf_filter_file = NULL;
+    } else {
+#ifdef HAVE_PACKET_EBPF
+        SCLogConfig("af-packet will use '%s' as eBPF filter file",
+                  ebpf_file);
+#endif
+        aconf->ebpf_filter_file = ebpf_file;
+        ConfGetChildValueBoolWithDefault(if_root, if_default, "bypass", &conf_val);
+        if (conf_val) {
+#ifdef HAVE_PACKET_EBPF
+            SCLogConfig("Using bypass kernel functionality for AF_PACKET (iface %s)",
+                    aconf->iface);
+            aconf->flags |= AFP_BYPASS;
+            RunModeEnablesBypassManager();
+            BypassedFlowManagerRegisterCheckFunc(EBPFCheckBypassedFlowTimeout);
+            BypassedFlowManagerRegisterUpdateFunc(EBPFUpdateFlow);
+#else
+            SCLogError(SC_ERR_UNIMPLEMENTED, "Bypass set but eBPF support is not built-in");
+#endif
+        }
+    }
+
+    /* One shot loading of the eBPF file */
+    if (aconf->ebpf_filter_file) {
+#ifdef HAVE_PACKET_EBPF
+        int ret = EBPFLoadFile(aconf->iface, aconf->ebpf_filter_file, "filter",
+                               &aconf->ebpf_filter_fd, EBPF_SOCKET_FILTER);
+        if (ret != 0) {
+            SCLogWarning(SC_ERR_INVALID_VALUE,
+                         "Error when loading eBPF filter file");
+        }
+#else
+        SCLogError(SC_ERR_UNIMPLEMENTED, "eBPF support is not build-in");
+#endif
+    }
+
+    if (ConfGetChildValueWithDefault(if_root, if_default, "xdp-filter-file", &ebpf_file) != 1) {
+        aconf->xdp_filter_file = NULL;
+    } else {
+        SCLogInfo("af-packet will use '%s' as XDP filter file",
+                  ebpf_file);
+        aconf->xdp_filter_file = ebpf_file;
+        ConfGetChildValueBoolWithDefault(if_root, if_default, "bypass", &conf_val);
+        if (conf_val) {
+#ifdef HAVE_PACKET_XDP
+            SCLogConfig("Using bypass kernel functionality for AF_PACKET (iface %s)",
+                    aconf->iface);
+            aconf->flags |= AFP_XDPBYPASS;
+            RunModeEnablesBypassManager();
+            BypassedFlowManagerRegisterCheckFunc(EBPFCheckBypassedFlowTimeout);
+#else
+            SCLogError(SC_ERR_UNIMPLEMENTED, "Bypass set but XDP support is not built-in");
+#endif
+        }
+#ifdef HAVE_PACKET_XDP
+        const char *xdp_mode;
+        if (ConfGetChildValueWithDefault(if_root, if_default, "xdp-mode", &xdp_mode) != 1) {
+            aconf->xdp_mode = XDP_FLAGS_SKB_MODE;
+        } else {
+            if (!strcmp(xdp_mode, "soft")) {
+                aconf->xdp_mode = XDP_FLAGS_SKB_MODE;
+            } else if (!strcmp(xdp_mode, "driver")) {
+                aconf->xdp_mode = XDP_FLAGS_DRV_MODE;
+            } else if (!strcmp(xdp_mode, "hw")) {
+                aconf->xdp_mode = XDP_FLAGS_HW_MODE;
+            } else {
+                SCLogWarning(SC_ERR_INVALID_VALUE,
+                             "Invalid xdp-mode value: '%s'", xdp_mode);
+            }
+        }
+#endif
+    }
+
+    /* One shot loading of the eBPF file */
+    if (aconf->xdp_filter_file) {
+#ifdef HAVE_PACKET_XDP
+        int ret = EBPFLoadFile(aconf->iface, aconf->xdp_filter_file, "xdp",
+                               &aconf->xdp_filter_fd, EBPF_XDP_CODE);
+        if (ret != 0) {
+            SCLogWarning(SC_ERR_INVALID_VALUE,
+                         "Error when loading XDP filter file");
+        } else {
+            ret = EBPFSetupXDP(aconf->iface, aconf->xdp_filter_fd, aconf->xdp_mode);
+            if (ret != 0) {
+                SCLogWarning(SC_ERR_INVALID_VALUE,
+                             "Error when setting up XDP");
+            } else {
+                /* Try to get the xdp-cpu-redirect key */
+                const char *cpuset;
+                if (ConfGetChildValueWithDefault(if_root, if_default,
+                                                 "xdp-cpu-redirect", &cpuset) == 1) {
+                    SCLogConfig("Setting up CPU map XDP");
+                    ConfNode *node = ConfGetChildWithDefault(if_root, if_default, "xdp-cpu-redirect");
+                    if (node == NULL) {
+                        SCLogError(SC_ERR_INVALID_VALUE, "Should not be there");
+                    } else {
+                        EBPFBuildCPUSet(node, aconf->iface);
+                    }
+                } else {
+                        /* It will just set CPU count to 0 */
+                        EBPFBuildCPUSet(NULL, aconf->iface);
+                }
+            }
+            /* we have a peer and we use bypass so we can set up XDP iface redirect */
+            if (aconf->out_iface) {
+                EBPFSetPeerIface(aconf->iface, aconf->out_iface);
+            }
+        }
+#else
+        SCLogError(SC_ERR_UNIMPLEMENTED, "XDP support is not built-in");
+#endif
+    }
+
     if ((ConfGetChildValueIntWithDefault(if_root, if_default, "buffer-size", &value)) == 1) {
         aconf->buffer_size = value;
     } else {
