@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Open Information Security Foundation
+/* Copyright (C) 2012-2017 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -22,11 +22,15 @@
  *  \author Matt Keeler <mk@npulsetech.com>
  */
 
+
 #ifndef __RUNMODE_NAPATECH_H__
 #define __RUNMODE_NAPATECH_H__
 
 #ifdef HAVE_NAPATECH
+#include "util-napatech.h"
 #include <nt.h>
+
+
 #endif
 
 int RunModeNapatechAutoFp(void);
@@ -34,4 +38,8 @@ int RunModeNapatechWorkers(void);
 void RunModeNapatechRegister(void);
 const char *RunModeNapatechGetDefaultMode(void);
 
+uint16_t GetNumConfiguredStreams(void);
+
+
+
 #endif /* __RUNMODE_NAPATECH_H__ */
