@@ -28,6 +28,6 @@ int RunModeIdsNetmapAutoFp(void);
 int RunModeIdsNetmapWorkers(void);
 void RunModeIdsNetmapRegister(void);
 const char *RunModeNetmapGetDefaultMode(void);
-int NetmapRunModeIsIPS();
+int NetmapRunModeIsIPS(void);
 
 #endif /* __RUNMODE_NETMAP_H__ */
