@@ -33,10 +33,11 @@
 #include "util-privs.h"
 #include "util-device.h"
 #include "tmqh-packetpool.h"
+#include "source-erf-dag.h"
 
 #ifndef HAVE_DAG
 
-TmEcode NoErfDagSupportExit(ThreadVars *, void *, void **);
+TmEcode NoErfDagSupportExit(ThreadVars *, const void *, void **);
 
 void
 TmModuleReceiveErfDagRegister(void)
@@ -65,7 +66,7 @@ TmModuleDecodeErfDagRegister(void)
 }
 
 TmEcode
-NoErfDagSupportExit(ThreadVars *tv, void *initdata, void **data)
+NoErfDagSupportExit(ThreadVars *tv, const void *initdata, void **data)
 {
     SCLogError(SC_ERR_DAG_NOSUPPORT,
         "Error creating thread %s: you do not have support for DAG cards "
@@ -75,7 +76,6 @@ NoErfDagSupportExit(ThreadVars *tv, void *initdata, void **data)
 
 #else /* Implied we do have DAG support */
 
-#include "source-erf-dag.h"
 #include <dagapi.h>
 
 /* Minimum amount of data to read from the DAG at a time. */
@@ -418,7 +418,7 @@ ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top, uint32_t *pkts_read)
 
         prec = (char *)ewtn->btm;
         dr = (dag_record_t*)prec;
-        rlen = ntohs(dr->rlen);
+        rlen = SCNtohs(dr->rlen);
         hdr_type = dr->type;
 
         /* If we don't have enough data to finish processing this ERF
@@ -443,7 +443,7 @@ ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top, uint32_t *pkts_read)
             break;
         case TYPE_ETH:
             if (dr->lctr) {
-                StatsAddUI64(ewtn->tv, ewtn->drops, ntohs(dr->lctr));
+                StatsAddUI64(ewtn->tv, ewtn->drops, SCNtohs(dr->lctr));
             }
             break;
         default:
@@ -482,8 +482,8 @@ ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec)
     Packet *p;
 
     hdr_type = dr->type;
-    wlen = ntohs(dr->wlen);
-    rlen = ntohs(dr->rlen);
+    wlen = SCNtohs(dr->wlen);
+    rlen = SCNtohs(dr->rlen);
 
     /* count extension headers */
     while (hdr_type & 0x80) {
