@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 Open Information Security Foundation
+/* Copyright (C) 2012-2017 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -18,10 +18,9 @@
 /**
  * \file
  *
- * \author nPulse Technologies, LLC
+ * \author nPulse Technologies, LLC.
  * \author Matt Keeler <mk@npulsetech.com>
  */
-
 #ifndef __SOURCE_NAPATECH_H__
 #define __SOURCE_NAPATECH_H__
 
@@ -29,16 +28,14 @@ void TmModuleNapatechStreamRegister (void);
 TmEcode NapatechStreamThreadDeinit(ThreadVars *tv, void *data);
 void TmModuleNapatechDecodeRegister (void);
 
+#ifdef HAVE_NAPATECH
+#include <nt.h>
+
 struct NapatechStreamDevConf
 {
-    int stream_id;
+    uint16_t stream_id;
     intmax_t hba;
 };
 
-#ifdef HAVE_NAPATECH
-
-#include <nt.h>
-
-#endif
-
+#endif /* HAVE_NAPATECH */
 #endif /* __SOURCE_NAPATECH_H__ */
