@@ -45,6 +45,10 @@
 #define CLS 64
 #endif
 
+#if HAVE_DIRENT_H
+#include <dirent.h>
+#endif
+
 #if HAVE_STDIO_H
 #include <stdio.h>
 #endif
@@ -184,6 +188,7 @@
 #include <netdb.h>
 #endif
 
+#ifndef SC_PCAP_DONT_INCLUDE_PCAP_H
 #ifdef HAVE_PCAP_H
 #include <pcap.h>
 #endif
@@ -191,9 +196,10 @@
 #ifdef HAVE_PCAP_PCAP_H
 #include <pcap/pcap.h>
 #endif
+#endif
 
-#ifdef HAVE_PCAP_BPF_H
-#include <pcap/bpf.h>
+#ifdef HAVE_UTIME_H
+#include <utime.h>
 #endif
 
 #ifdef HAVE_LIBGEN_H
@@ -206,6 +212,15 @@
 #endif
 #endif
 
+#if !__CYGWIN__
+#ifdef HAVE_WINSOCK2_H
+#include <winsock2.h>
+#endif
+#ifdef HAVE_WS2TCPIP_H
+#include <ws2tcpip.h>
+#endif
+#endif /* !__CYGWIN__ */
+
 #ifdef HAVE_WINDOWS_H
 #ifndef _WIN32_WINNT
 #define _WIN32_WINNT 0x0501
@@ -221,15 +236,6 @@
 #include <w32api/wtypes.h>
 #endif
 
-#if !__CYGWIN__
-#ifdef HAVE_WINSOCK2_H
-#include <winsock2.h>
-#endif
-#ifdef HAVE_WS2TCPIP_H
-#include <ws2tcpip.h>
-#endif
-#endif /* !__CYGWIN__ */
-
 #ifdef HAVE_LIBJANSSON
 #include <jansson.h>
 #ifndef JSON_ESCAPE_SLASH
@@ -367,13 +373,15 @@
 
 #define WARN_UNUSED __attribute__((warn_unused_result))
 
+#define SCNtohl(x) (uint32_t)ntohl((x))
+#define SCNtohs(x) (uint16_t)ntohs((x))
+
 typedef enum PacketProfileDetectId_ {
+    PROF_DETECT_SETUP,
+    PROF_DETECT_GETSGH,
     PROF_DETECT_IPONLY,
     PROF_DETECT_RULES,
-    PROF_DETECT_STATEFUL_START,
-    PROF_DETECT_STATEFUL_CONT,
-    PROF_DETECT_STATEFUL_UPDATE,
-    PROF_DETECT_PREFILTER,
+    PROF_DETECT_TX,
     PROF_DETECT_PF_PKT,
     PROF_DETECT_PF_PAYLOAD,
     PROF_DETECT_PF_TX,
@@ -381,15 +389,39 @@ typedef enum PacketProfileDetectId_ {
     PROF_DETECT_PF_SORT2,
     PROF_DETECT_NONMPMLIST,
     PROF_DETECT_ALERT,
+    PROF_DETECT_TX_UPDATE,
     PROF_DETECT_CLEANUP,
-    PROF_DETECT_GETSGH,
-    PROF_DETECT_MPM_FD_SMTP,
 
     PROF_DETECT_SIZE,
 } PacketProfileDetectId;
 
+/** \note update PacketProfileLoggertIdToString if you change anything here */
 typedef enum {
     LOGGER_UNDEFINED,
+
+    /* TX loggers first for low logger IDs */
+    LOGGER_DNS_TS,
+    LOGGER_DNS_TC,
+    LOGGER_HTTP,
+    LOGGER_TLS_STORE,
+    LOGGER_TLS,
+    LOGGER_JSON_DNS_TS,
+    LOGGER_JSON_DNS_TC,
+    LOGGER_JSON_HTTP,
+    LOGGER_JSON_SMTP,
+    LOGGER_JSON_TLS,
+    LOGGER_JSON_NFS,
+    LOGGER_JSON_TFTP,
+    LOGGER_JSON_DNP3_TS,
+    LOGGER_JSON_DNP3_TC,
+    LOGGER_JSON_SSH,
+    LOGGER_JSON_SMB,
+    LOGGER_JSON_IKEV2,
+    LOGGER_JSON_KRB5,
+    LOGGER_JSON_DHCP,
+    LOGGER_JSON_TEMPLATE_RUST,
+    LOGGER_JSON_TEMPLATE,
+
     LOGGER_ALERT_DEBUG,
     LOGGER_ALERT_FAST,
     LOGGER_UNIFIED2,
@@ -397,17 +429,6 @@ typedef enum {
     LOGGER_DROP,
     LOGGER_JSON_ALERT,
     LOGGER_JSON_DROP,
-    LOGGER_JSON_SSH,
-    LOGGER_DNS,
-    LOGGER_HTTP,
-    LOGGER_JSON_DNS,
-    LOGGER_JSON_HTTP,
-    LOGGER_JSON_SMTP,
-    LOGGER_JSON_TLS,
-    LOGGER_JSON_NFS,
-    LOGGER_JSON_TEMPLATE,
-    LOGGER_TLS_STORE,
-    LOGGER_TLS,
     LOGGER_FILE,
     LOGGER_FILE_STORE,
     LOGGER_JSON_FILE,
@@ -418,11 +439,11 @@ typedef enum {
     LOGGER_JSON_STATS,
     LOGGER_PRELUDE,
     LOGGER_PCAP,
-    LOGGER_JSON_DNP3,
-    LOGGER_JSON_VARS,
+    LOGGER_JSON_METADATA,
     LOGGER_SIZE,
 } LoggerId;
 
+#include "util-optimize.h"
 #include <htp/htp.h>
 #include "threads.h"
 #include "tm-threads-common.h"
@@ -430,7 +451,6 @@ typedef enum {
 #include "util-error.h"
 #include "util-mem.h"
 #include "detect-engine-alert.h"
-#include "util-optimize.h"
 #include "util-path.h"
 #include "util-conf.h"
 
@@ -446,6 +466,9 @@ size_t strlcat(char *, const char *src, size_t siz);
 #ifndef HAVE_STRLCPY
 size_t strlcpy(char *dst, const char *src, size_t siz);
 #endif
+#ifndef HAVE_STRPTIME
+char *strptime(const char * __restrict, const char * __restrict, struct tm * __restrict);
+#endif
 
 extern int coverage_unittests;
 extern int g_ut_modules;
