@@ -229,10 +229,14 @@ const char * TmModuleTmmIdToString(TmmId id)
         CASE_CODE (TMM_STATSLOGGER);
         CASE_CODE (TMM_FLOWMANAGER);
         CASE_CODE (TMM_FLOWRECYCLER);
+        CASE_CODE (TMM_BYPASSEDFLOWMANAGER);
         CASE_CODE (TMM_UNIXMANAGER);
         CASE_CODE (TMM_DETECTLOADER);
         CASE_CODE (TMM_RECEIVENETMAP);
         CASE_CODE (TMM_DECODENETMAP);
+        CASE_CODE (TMM_RECEIVEWINDIVERT);
+        CASE_CODE (TMM_VERDICTWINDIVERT);
+        CASE_CODE (TMM_DECODEWINDIVERT);
 
         CASE_CODE (TMM_SIZE);
     }
