@@ -29,19 +29,16 @@ enum {
     TMQH_NFQ,
     TMQH_PACKETPOOL,
     TMQH_FLOW,
-    TMQH_RINGBUFFER_MRSW,
-    TMQH_RINGBUFFER_SRSW,
-    TMQH_RINGBUFFER_SRMW,
 
     TMQH_SIZE,
 };
 
 typedef struct Tmqh_ {
-    char *name;
+    const char *name;
     Packet *(*InHandler)(ThreadVars *);
     void (*InShutdownHandler)(ThreadVars *);
     void (*OutHandler)(ThreadVars *, Packet *);
-    void *(*OutHandlerCtxSetup)(char *);
+    void *(*OutHandlerCtxSetup)(const char *);
     void (*OutHandlerCtxFree)(void *);
     void (*RegisterTests)(void);
 } Tmqh;
@@ -50,7 +47,7 @@ Tmqh tmqh_table[TMQH_SIZE];
 
 void TmqhSetup (void);
 void TmqhCleanup(void);
-Tmqh* TmqhGetQueueHandlerByName(char *name);
+Tmqh* TmqhGetQueueHandlerByName(const char *name);
 
 #endif /* __TM_QUEUEHANDLERS_H__ */
 
