@@ -61,9 +61,13 @@ typedef enum {
     TMM_STATSLOGGER,
     TMM_RECEIVENFLOG,
     TMM_DECODENFLOG,
+    TMM_RECEIVEWINDIVERT,
+    TMM_VERDICTWINDIVERT,
+    TMM_DECODEWINDIVERT,
 
     TMM_FLOWMANAGER,
     TMM_FLOWRECYCLER,
+    TMM_BYPASSEDFLOWMANAGER,
     TMM_DETECTLOADER,
 
     TMM_UNIXMANAGER,
