@@ -66,7 +66,7 @@ static int SCAtomicTest01(void)
 void SCAtomicRegisterTests(void)
 {
 #ifdef UNITTESTS
-    UtRegisterTest("SCAtomicTest01", SCAtomicTest01, 1);
+    UtRegisterTest("SCAtomicTest01", SCAtomicTest01);
 #endif
 
     return;
