@@ -240,7 +240,7 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_AFP_DISPATCH);
         CASE_CODE (SC_ERR_CMD_LINE);
         CASE_CODE (SC_ERR_SIZE_PARSE);
-        CASE_CODE (SC_ERR_RAWBYTES_FILE_DATA);
+        CASE_CODE (SC_ERR_RAWBYTES_BUFFER);
         CASE_CODE (SC_ERR_SOCKET);
         CASE_CODE (SC_ERR_PCAP_TRANSLATE);
         CASE_CODE (SC_WARN_OUTDATED_LIBHTP);
@@ -342,7 +342,23 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_WARN_EVENT_DROPPED);
         CASE_CODE (SC_ERR_NO_REDIS_ASYNC);
         CASE_CODE (SC_ERR_REDIS_CONFIG);
+        CASE_CODE (SC_ERR_BYPASS_NOT_SUPPORTED);
+        CASE_CODE (SC_WARN_RENAMING_FILE);
         CASE_CODE (SC_ERR_PF_RING_VLAN);
+        CASE_CODE (SC_ERR_CREATE_DIRECTORY);
+        CASE_CODE (SC_WARN_FLOWBIT);
+        CASE_CODE (SC_ERR_SMB_CONFIG);
+        CASE_CODE (SC_WARN_NO_JA3_SUPPORT);
+        CASE_CODE (SC_WARN_JA3_DISABLED);
+        CASE_CODE (SC_ERR_PCAP_LOG_COMPRESS);
+        CASE_CODE (SC_ERR_FSEEK);
+        CASE_CODE (SC_ERR_WINDIVERT_GENERIC);
+        CASE_CODE (SC_ERR_WINDIVERT_NOSUPPORT);
+        CASE_CODE (SC_ERR_WINDIVERT_INVALID_FILTER);
+        CASE_CODE (SC_ERR_WINDIVERT_TOOLONG_FILTER);
+        CASE_CODE (SC_WARN_RUST_NOT_AVAILABLE);
+
+        CASE_CODE (SC_ERR_MAX);
     }
 
     return "UNKNOWN_ERROR";
