@@ -228,7 +228,7 @@ typedef enum {
     SC_ERR_MAGIC_OPEN,
     SC_ERR_MAGIC_LOAD,
     SC_ERR_SIZE_PARSE,
-    SC_ERR_RAWBYTES_FILE_DATA,
+    SC_ERR_RAWBYTES_BUFFER,
     SC_ERR_SOCKET,
     SC_ERR_PCAP_TRANSLATE,          /* failed to translate ip to dev */
     SC_WARN_OUTDATED_LIBHTP,
@@ -332,7 +332,23 @@ typedef enum {
     SC_WARN_EVENT_DROPPED,
     SC_ERR_NO_REDIS_ASYNC,
     SC_ERR_REDIS_CONFIG,
+    SC_ERR_BYPASS_NOT_SUPPORTED,
+    SC_WARN_RENAMING_FILE,
     SC_ERR_PF_RING_VLAN,
+    SC_ERR_CREATE_DIRECTORY,
+    SC_WARN_FLOWBIT,
+    SC_ERR_SMB_CONFIG,
+    SC_WARN_NO_JA3_SUPPORT,
+    SC_WARN_JA3_DISABLED,
+    SC_ERR_PCAP_LOG_COMPRESS,
+    SC_ERR_FSEEK,
+    SC_ERR_WINDIVERT_GENERIC,
+    SC_ERR_WINDIVERT_NOSUPPORT,
+    SC_ERR_WINDIVERT_INVALID_FILTER,
+    SC_ERR_WINDIVERT_TOOLONG_FILTER,
+    SC_WARN_RUST_NOT_AVAILABLE,
+
+    SC_ERR_MAX,
 } SCError;
 
 const char *SCErrorToString(SCError);
