@@ -179,7 +179,6 @@ PktProfiling *SCProfilePacketStart(void);
 
 #define PACKET_PROFILING_RESET(p)                                   \
     if (profiling_packets_enabled && (p)->profile != NULL) {        \
-        SCFree((p)->profile->prefilter.engines);                    \
         SCFree((p)->profile);                                       \
         (p)->profile = NULL;                                        \
     }
@@ -275,34 +274,30 @@ PktProfiling *SCProfilePacketStart(void);
         SCProfilingSghUpdateCounter((det_ctx), (sgh));              \
     }
 
-#define PROFILING_PREFILTER_RESET(p, detectsize) \
-    if (profiling_packets_enabled  && (p)->profile != NULL) {       \
-        if ((p)->profile->prefilter.size != ((detectsize) + 1)) {   \
-            if ((p)->profile->prefilter.engines != NULL)            \
-                SCFree((p)->profile->prefilter.engines);            \
-            (p)->profile->prefilter.engines =                       \
-                SCCalloc((detectsize)+1, sizeof(PktProfilingPrefilterEngine)); \
-            (p)->profile->prefilter.size = (detectsize)+1;\
-        } else {                                                    \
-            memset((p)->profile->prefilter.engines, 0x00,           \
-                ((detectsize)+1 * sizeof(PktProfilingPrefilterEngine))); \
-        }                                                           \
-    }                                                               \
+extern int profiling_prefilter_enabled;
+extern __thread int profiling_prefilter_entered;
 
-#define PROFILING_PREFILTER_START(p) \
-    uint64_t ticks_start = 0; \
-    if (profiling_packets_enabled  && (p)->profile != NULL) {       \
-        ticks_start = UtilCpuGetTicks();                            \
-    }                                                               \
+#define PREFILTER_PROFILING_START \
+    uint64_t profile_prefilter_start_ = 0; \
+    uint64_t profile_prefilter_end_ = 0; \
+    if (profiling_prefilter_enabled) { \
+        if (profiling_prefilter_entered > 0) { \
+            SCLogError(SC_ERR_FATAL, "Re-entered profiling, exiting."); \
+            abort(); \
+        } \
+        profiling_prefilter_entered++; \
+        profile_prefilter_start_ = UtilCpuGetTicks(); \
+    }
 
-#define PROFILING_PREFILTER_END(p, profile_id) \
-    if (profiling_packets_enabled && (p)->profile != NULL &&        \
-        ticks_start)                                                \
-    {                                                               \
-        uint64_t ticks_end = UtilCpuGetTicks();                     \
-        (p)->profile->prefilter.engines[(profile_id)].ticks_spent += (ticks_end - ticks_start);    \
-        ticks_start = 0;                                            \
-    }                                                               \
+/* we allow this macro to be called if profiling_prefilter_entered == 0,
+ * so that we don't have to refactor some of the detection code. */
+#define PREFILTER_PROFILING_END(ctx, profile_id) \
+    if (profiling_prefilter_enabled && profiling_prefilter_entered) { \
+        profile_prefilter_end_ = UtilCpuGetTicks(); \
+        if (profile_prefilter_end_ > profile_prefilter_start_) \
+            SCProfilingPrefilterUpdateCounter((ctx),(profile_id),(profile_prefilter_end_ - profile_prefilter_start_)); \
+        profiling_prefilter_entered--; \
+    }
 
 void SCProfilingRulesGlobalInit(void);
 void SCProfilingRuleDestroyCtx(struct SCProfileDetectCtx_ *);
@@ -318,6 +313,14 @@ void SCProfilingKeywordUpdateCounter(DetectEngineThreadCtx *det_ctx, int id, uin
 void SCProfilingKeywordThreadSetup(struct SCProfileKeywordDetectCtx_ *, DetectEngineThreadCtx *);
 void SCProfilingKeywordThreadCleanup(DetectEngineThreadCtx *);
 
+struct SCProfilePrefilterDetectCtx_;
+void SCProfilingPrefilterGlobalInit(void);
+void SCProfilingPrefilterDestroyCtx(DetectEngineCtx *);
+void SCProfilingPrefilterInitCounters(DetectEngineCtx *);
+void SCProfilingPrefilterUpdateCounter(DetectEngineThreadCtx *det_ctx, int id, uint64_t ticks);
+void SCProfilingPrefilterThreadSetup(struct SCProfilePrefilterDetectCtx_ *, DetectEngineThreadCtx *);
+void SCProfilingPrefilterThreadCleanup(DetectEngineThreadCtx *);
+
 void SCProfilingSghsGlobalInit(void);
 void SCProfilingSghDestroyCtx(DetectEngineCtx *);
 void SCProfilingSghInitCounters(DetectEngineCtx *);
@@ -367,9 +370,8 @@ void SCProfilingDump(void);
 #define FLOWWORKER_PROFILING_START(p, id)
 #define FLOWWORKER_PROFILING_END(p, id)
 
-#define PROFILING_PREFILTER_RESET(p, detectsize)
-#define PROFILING_PREFILTER_START(p)
-#define PROFILING_PREFILTER_END(p, profile_id)
+#define PREFILTER_PROFILING_START
+#define PREFILTER_PROFILING_END(ctx, profile_id)
 
 #endif /* PROFILING */
 
