@@ -303,12 +303,13 @@ static void PreBmGsNocase(const uint8_t *x, uint16_t m, uint16_t *bmGs)
  *
  * \retval ptr to start of the match; NULL if no match
  */
-uint8_t *BoyerMoore(const uint8_t *x, uint16_t m, const uint8_t *y, int32_t n, BmCtx *bm_ctx)
+uint8_t *BoyerMoore(const uint8_t *x, uint16_t m, const uint8_t *y, uint32_t n, BmCtx *bm_ctx)
 {
     uint16_t *bmGs = bm_ctx->bmGs;
     uint16_t *bmBc = bm_ctx->bmBc;
 
-   int i, j, m1, m2;
+    int i, j, m1, m2;
+    int32_t int_n;
 #if 0
     printf("\nBad:\n");
     for (i=0;i<ALPHABET_SIZE;i++)
@@ -319,21 +320,22 @@ uint8_t *BoyerMoore(const uint8_t *x, uint16_t m, const uint8_t *y, int32_t n, B
         printf("%c, %d ", x[i],bmBc[i]);
     printf("\n");
 #endif
-   j = 0;
+    // force casting to int32_t (if possible)
+    int_n = unlikely(n > INT32_MAX) ? INT32_MAX : n;
+    j = 0;
+    while (j <= int_n - m ) {
+        for (i = m - 1; i >= 0 && x[i] == y[i + j]; --i);
 
-   while (j <= n - m ) {
-      for (i = m - 1; i >= 0 && x[i] == y[i + j]; --i);
-
-      if (i < 0) {
-         return (uint8_t *)(y + j);
-         //j += bmGs[0];
-      } else {
- //        printf("%c", y[i+j]);
-         j += (m1 = bmGs[i]) > (m2 = bmBc[y[i + j]] - m + 1 + i)? m1: m2;
-//            printf("%d, %d\n", m1, m2);
-      }
-   }
-   return NULL;
+        if (i < 0) {
+            return (uint8_t *)(y + j);
+            //j += bmGs[0];
+        } else {
+//          printf("%c", y[i+j]);
+            j += (m1 = bmGs[i]) > (m2 = bmBc[y[i + j]] - m + 1 + i)? m1: m2;
+//          printf("%d, %d\n", m1, m2);
+        }
+    }
+    return NULL;
 }
 
 
@@ -352,11 +354,12 @@ uint8_t *BoyerMoore(const uint8_t *x, uint16_t m, const uint8_t *y, int32_t n, B
  *
  * \retval ptr to start of the match; NULL if no match
  */
-uint8_t *BoyerMooreNocase(const uint8_t *x, uint16_t m, const uint8_t *y, int32_t n, BmCtx *bm_ctx)
+uint8_t *BoyerMooreNocase(const uint8_t *x, uint16_t m, const uint8_t *y, uint32_t n, BmCtx *bm_ctx)
 {
     uint16_t *bmGs = bm_ctx->bmGs;
     uint16_t *bmBc = bm_ctx->bmBc;
     int i, j, m1, m2;
+    int32_t int_n;
 #if 0
     printf("\nBad:\n");
     for (i=0;i<ALPHABET_SIZE;i++)
@@ -367,18 +370,21 @@ uint8_t *BoyerMooreNocase(const uint8_t *x, uint16_t m, const uint8_t *y, int32_
         printf("%c, %d ", x[i],bmBc[i]);
     printf("\n");
 #endif
+    // force casting to int32_t (if possible)
+    int_n = unlikely(n > INT32_MAX) ? INT32_MAX : n;
     j = 0;
-    while (j <= n - m ) {
-        /* x is stored in lowercase. */
+    while (j <= int_n - m ) {
+         /* x is stored in lowercase. */
         for (i = m - 1; i >= 0 && x[i] == u8_tolower(y[i + j]); --i);
 
         if (i < 0) {
             return (uint8_t *)(y + j);
         } else {
-            j += (m1=bmGs[i]) > (m2=bmBc[u8_tolower(y[i + j])] - m + 1 + i)?m1:m2;
+            j += (m1 = bmGs[i]) > (m2 = bmBc[u8_tolower(y[i + j])] - m + 1 + i)?
+                m1: m2;
         }
-   }
-   return NULL;
+    }
+    return NULL;
 }
 
 typedef struct SpmBmCtx_ {
@@ -448,7 +454,7 @@ static void BMDestroyCtx(SpmCtx *ctx)
 }
 
 static uint8_t *BMScan(const SpmCtx *ctx, SpmThreadCtx *thread_ctx,
-                       const uint8_t *haystack, uint16_t haystack_len)
+                       const uint8_t *haystack, uint32_t haystack_len)
 {
     const SpmBmCtx *sctx = ctx->ctx;
 
