@@ -430,8 +430,10 @@ time_t SCMkTimeUtc (struct tm *tp)
     result += tp->tm_min;
     result *= 60;
     result += tp->tm_sec;
+#ifndef OS_WIN32
     if (tp->tm_gmtoff)
         result -= tp->tm_gmtoff;
+#endif
     return result;
 }
 
@@ -463,8 +465,10 @@ int SCStringPatternToTime (char *string, const char **patterns, int num_patterns
         tp->tm_hour = tp->tm_min = tp->tm_sec = 0;
         tp->tm_year = tp->tm_mon = tp->tm_mday = tp->tm_wday = INT_MIN;
         tp->tm_isdst = -1;
+#ifndef OS_WIN32
         tp->tm_gmtoff = 0;
         tp->tm_zone = NULL;
+#endif
         result = strptime(string, patterns[i], tp);
 
         if (result && *result == '\0')
@@ -599,3 +603,8 @@ uint64_t SCGetSecondsUntil (const char *str, time_t epoch)
 
     return seconds;
 }
+
+uint64_t SCTimespecAsEpochMillis(const struct timespec* ts)
+{
+    return ts->tv_sec * 1000L + ts->tv_nsec / 1000000L;
+}
