@@ -24,6 +24,8 @@
 #ifndef __WIN32_MISC_H__
 #define __WIN32_MISC_H__
 
+#include <sdkddkver.h>
+
 #define index strchr
 #define rindex strrchr
 
@@ -36,8 +38,10 @@
 void setenv(const char *name, const char *value, int overwrite);
 void unsetenv(const char *name);
 
+#if NTDDI_VERSION < NTDDI_VISTA
 const char* inet_ntop(int af, const void *src, char *dst, uint32_t cnt);
 int inet_pton(int af, const char *src, void *dst);
+#endif
 
 #define geteuid() (0)
 
